module NfseCarioca
  class CancelarNfse
    attr_reader :invoice_number

    def initialize(invoice_number)
      @invoice_number = invoice_number
    end

    def execute!
      client.call(:cancelar_nfse, self)
    end

    def to_xml
      xml = Builder::XmlMarkup.new(indent: 2)
      xml.instruct!
      xml.CancelarNfseEnvio(xmlns: "http://www.abrasf.org.br/ABRASF/arquivos/nfse.xsd") do |cancelar_nfse_envio|
        cancelar_nfse_envio.Pedido(xmlns: "http://www.abrasf.org.br/ABRASF/arquivos/nfse.xsd") do |pedido|
          pedido.InfPedidoCancelamento(Id: "Cancelamento_NF#{invoice_number}") do |inf_pedido_cancelamento|
            inf_pedido_cancelamento.IdentificacaoNfse do |identificacao_nfse|
              identificacao_nfse.Numero invoice_number
              identificacao_nfse.Cnpj NfseCarioca.configuration.cnpj
              identificacao_nfse.InscricaoMunicipal NfseCarioca.configuration.inscricao_municipal
              identificacao_nfse.CodigoMunicipio NfseCarioca.configuration.codigo_municipio
            end
            inf_pedido_cancelamento.CodigoCancelamento "0"
          end
        end
      end
    end

    private
    def client
      NfseCarioca::Client.new
    end

    class Response < NfseCarioca::Response
      def error_message
        content["ListaMensagemRetorno"]["MensagemRetorno"]["Correcao"] if error?
      end
    end
  end
end
